/****************************************************************/
/*** Simulates d-dim percolation                              ***/
/*** A.K. Hartmann April 2005                                 ***/
/*** Statistische Mechanik ungeordneter Systeme               ***/
/*** University of Goettingen, Germany 2005                   ***/
/****************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "percol.h"

int main(int argc, char *argv[])
{
  int dim;                    /* dimension of system */
  int num_n;    /* number of neighbours of each site */
  int *l;                         /* sizes of system */
  int *next;           /* stores neighbours of sites */
  int t, t2, r, d;                  /* loop counters */
  int N;                          /* number of sites */
  int id;                            /* id of a site */
  int *occupy;             /* ids of sites to occupy */
  int seed;            /* of random number generator */
  int run, num_runs;   /* number of independent runs */
  int last_percol; /* value of flag at last iteration*/   
  double *size_max, *percol_prob;
  percol_glob_t *glob;         /* global system data */
  percol_node_t **node;            /* occupied nodes */

  seed = 1000;
  dim = 2;                     /* initialization + parameters */
  num_n = 2*dim;                            /* simple lattice */
  l = (int *) malloc(dim*sizeof(int));

  if(argc<3)
  {
    printf("USAGE %s <num_runs> <L>\n", argv[0]);
    exit(1);
  }
  num_runs = atoi(argv[1]);
  l[0] = atoi(argv[2]);
  for(d=1; d<dim; d++)
    l[d] = l[0];

  next = percol_init_next(dim, l, &N);
  glob = percol_create_glob(dim, l);

  /*for(t=1; t<=N; t++)
    for(r=0; r<2*dim; r++)
    printf("NEXT(%d,%d)=%d\n",t, r, NEXT(t,r));
  exit(1);*/
  size_max = (double *) malloc((N+1)*sizeof(double));
  percol_prob = (double *) malloc((N+1)*sizeof(double));
  occupy = (int *) malloc((N+1)*sizeof(int));
  node = (percol_node_t **) 
    malloc((N+1)*sizeof(percol_node_t *));
  for(t=1; t<=N; t++)                  /* initialize statistics */
  {
    size_max[t] = 0.0;
    percol_prob[t] = 0.0;
  }

  for(run=0; run<num_runs; run++)                  /* main loop */
  {
    srand48(seed++);
    glob->largest = 0;
    for(d=0; d<glob->dim; d++)
      glob->spanning[d] = 0;
    last_percol = 0;
    for(t=1; t<=N; t++)
    {
      occupy[t] = t;
      node[t] = NULL;             /* initially lattice is empty */
    }
    for(t=1; t<=N; t++)   /* randomize order sites are occupied */
    {
      t2 = t+(int)floor((N-t+1)*drand48());    /* one remaining */
      id = occupy[t];                 /* exchange elements t,t2 */
      occupy[t] = occupy[t2];
      occupy[t2] = id;
    }
    for(t=1; t<=N; t++)    /* occupy sites, one after the other */
    {
      percol_occupy_site(occupy[t], t, node, next, num_n, glob);
      size_max[t] += (double) glob->largest/N;
      percol_prob[t] += glob->spanning[0];
      /*if(last_percol != glob->spanning[0])
      {
	printf("ADDED: %d\n", occupy[t]);
	percol_print_cluster(node, glob);
	}*/
      last_percol = glob->spanning[0];
    }
    printf("# percol: %d\n", glob->spanning_n[0]);
    for(t=1; t<=N; t++)                  /* empty lattice again */
      if(node[t] != NULL)
	percol_delete_node(node[t]);
  }
  /*percol_print_cluster(node, glob);*/

  for(t=0; t<=N; t++)
    printf("%f %f %f\n", (double) t/N, size_max[t]/num_runs,
	   percol_prob[t]/num_runs);

  percol_delete_glob(glob);
  free(next);
  free(l);
  return(0);
}
